/*
    lm78.c - A Linux module for reading sensor data.
    Copyright (c) 1997, 1998  Alexander Larsson <alla@lysator.liu.se>,
    Frodo Looijaard <frodol@dds.nl> and Philip Edelbrock <phil@netroedge.com>.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#define LM78_VERSION  "0.01"

#include <linux/config.h>

/* Hack to allow the use of CONFIG_MODVERSIONS. In the kernel itself, this
   is arranged from the main Makefile. */
#if CONFIG_MODVERSIONS==1
#define MODVERSIONS
#include <linux/modversions.h>
#endif

#include <linux/module.h>
#include <linux/proc_fs.h>
#include "main.h"

static int LM75_Disabled = 0;

/* Must these be external (like they are now)?!? */
int init_module(void);
void cleanup_module(void);

#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,1,0))
static int lm78_read(char *buf, char **start, off_t offset,
                     int len, int *eof, void *private);
#else
static int lm78_read(char *, char **, off_t, int, int);
#endif

#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,1,0))
MODULE_AUTHOR("Alexander Larsson <alla@lysator.liu.se>, Frodo Looijaard <frodol@dds.nl>, Philip Edelbrock <phil@Ren.netroedge.com>");
MODULE_DESCRIPTION("Driver for reading sensor values from a lm78 on the motherboard.");
#endif

#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,1,0))
 static struct proc_dir_entry *proc_sensors = NULL;
#else
 static int inodeval = 0; /* Things depend on this initialization! */
 struct proc_dir_entry dir =
 {
   0, 7, "sensors", 
   S_IFREG | S_IRUGO, 1, 0, 0,
   0, NULL,
   &lm78_read
 };
#endif

/*
 *	Initialize the module.
 */
int init_module(void)
{
  int err;

  err=Init_LM78();
  if (err != 0) {
    printk("LM78 not found or busy.\n");
    return err;
  }
  printk("LM78 detected and installed.\n");

  if (LM75_Disabled == 0) {
    err=Init_LM75();
    if (err != 0) {
      printk("SMBus/LM75 not found or busy.\n");
      LM75_Disabled = 1;
    } else 
      printk("SMBus/LM75 detected and installed.\n");
  }

  /* register /proc/sensors */
#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,1,0))
  
  proc_sensors = create_proc_entry("sensors", 0, 0);
  if (proc_sensors) {
    proc_sensors->read_proc = lm78_read;
  } else {
    printk("Couldn't create /proc/sensors\n");
    cleanup_module();
    return -ENOENT;
  }
  
#else
  
/*
 *	Try to register the file in the root directory of the proc
 *	filesystem.  Dynamic registration means we won't provide a fixed
 *	inode number, but rather the registration code will allocate one for
 *	us.  This is safer, since we never know when someone will add a new
 *	fixed inode number to the filesystem.  Possible returns are 0
 *	(success) and -ENOENT (failure), and since there's only one error we
 *	won't worry unduly about messages.  If successful, continue.
 */
  if (proc_register_dynamic(&proc_root, &dir)) {
    printk("Couldn't create /proc/sensors\n");
    cleanup_module();
  }
  
/*
 *      Save the inode value from the registration call, as we will need
 *      it to unregister.
 */
  inodeval = dir.low_ino;

#endif
  
  return 0;
}

/*
 *	Remove the module.  If a nonzero inode value was returned,
 *	unregister the /proc directory entry; that's all we need to do.
 */

void cleanup_module(void)
{
	Cleanup_LM78();
        if (! LM75_Disabled)
                Cleanup_LM75();
      
#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,1,0))
        if (proc_sensors)
		proc_unregister(&proc_root, proc_sensors->low_ino);
#else
        if (inodeval)
		proc_unregister(&proc_root, inodeval);
#endif
}


/*
 * Ahem, gotta find out how this is supposed to work, ie. how big is 'buf' 
 */
#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,1,0))
static int lm78_read(char *buf, char **start, off_t offset, int len, int *eof, void *private)
#else
static int lm78_read(char *buf, char **start, off_t offset, int len, int unused)
#endif
{
  len=0;

  len = print_lm78_values(buf);
  if (! LM75_Disabled)
    len += print_lm75_values (buf+len);

#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,1,0))
  *start = buf + offset;
  return len > offset ? len - offset : 0;
#else
  return len;
#endif
}

